"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseDeclarations = void 0;
const expression_js_1 = require("./expression.js");
const util_js_1 = require("./util.js");
const values_js_1 = require("./values.js");
function parseDeclarations(ctx) {
    let pos = (0, util_js_1.whitespaces)(ctx.source, 0);
    const declarations = [];
    while (ctx.source.startsWith('let', pos)) {
        const decl = parseDeclaration(ctx, pos);
        declarations.push(decl);
        pos = decl.end;
        pos += (0, util_js_1.whitespaces)(ctx.source, pos);
    }
    checkLocalVarReferences(ctx, declarations);
    return { declarations, end: pos };
}
exports.parseDeclarations = parseDeclarations;
// declaration = let s variable [s] "=" [s] expression
// let = %x6C.65.74 ; "let"
function parseDeclaration(ctx, start) {
    let pos = start + 3; // 'let'
    const ws = (0, util_js_1.whitespaces)(ctx.source, pos);
    pos += ws;
    if (ws === 0)
        ctx.onError('missing-char', pos, ' ');
    let target;
    if (ctx.source[pos] === '$') {
        target = (0, values_js_1.parseVariable)(ctx, pos);
        pos = target.end;
    }
    else {
        const junkStart = pos;
        const junkEndOffset = ctx.source.substring(pos).search(/[\t\n\r ={}]/);
        pos = junkEndOffset === -1 ? ctx.source.length : pos + junkEndOffset;
        target = {
            type: 'junk',
            start: junkStart,
            end: pos,
            source: ctx.source.substring(junkStart, pos)
        };
        ctx.onError('missing-char', junkStart, '$');
    }
    pos += (0, util_js_1.whitespaces)(ctx.source, pos);
    if (ctx.source[pos] === '=')
        pos += 1;
    else
        ctx.onError('missing-char', pos, '=');
    let value;
    pos += (0, util_js_1.whitespaces)(ctx.source, pos);
    if (ctx.source[pos] === '{') {
        value = (0, expression_js_1.parseExpression)(ctx, pos);
        pos = value.end;
    }
    else {
        const junkStart = pos;
        const junkEndOffset = ctx.source
            .substring(pos)
            .search(/\blet|\bmatch|\bwhen|[${}]/);
        pos = junkEndOffset === -1 ? ctx.source.length : pos + junkEndOffset;
        value = {
            type: 'junk',
            start: junkStart,
            end: pos,
            source: ctx.source.substring(junkStart, pos)
        };
        ctx.onError('missing-char', junkStart, '{');
    }
    return { start, end: pos, target, value };
}
/** Local variable declarations can't refer to later ones */
function checkLocalVarReferences(ctx, declarations) {
    const check = (name, ref) => {
        if (ref.name === name) {
            ctx.onError('bad-local-var', ref.start, ref.end);
        }
    };
    for (let i = 1; i < declarations.length; ++i) {
        const { name } = declarations[i].target;
        if (!name)
            continue;
        for (let j = 0; j < i; ++j) {
            const ph = declarations[j].value;
            if (ph.type === 'expression') {
                const exp = ph.body;
                switch (exp.type) {
                    case 'function':
                        if (exp.operand?.type === 'variable')
                            check(name, exp.operand);
                        for (const opt of exp.options) {
                            if (opt.value.type === 'variable')
                                check(name, opt.value);
                        }
                        break;
                    case 'variable':
                        check(name, exp);
                        break;
                }
            }
        }
    }
}
