"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveFunctionRef = exports.functionRefSourceName = exports.isFunctionRef = void 0;
const errors_1 = require("../errors");
const message_value_1 = require("../message-value");
const message_value_2 = require("../message-value/message-value");
/**
 * Type guard for {@link FunctionRef} pattern elements
 *
 * @beta
 */
// eslint-disable-next-line @typescript-eslint/no-explicit-any
const isFunctionRef = (part) => !!part && typeof part === 'object' && part.type === 'function';
exports.isFunctionRef = isFunctionRef;
function functionRefSourceName(kind, name) {
    switch (kind) {
        case 'open':
            return `+${name}`;
        case 'close':
            return `-${name}`;
        default:
            return `:${name}`;
    }
}
exports.functionRefSourceName = functionRefSourceName;
function resolveFunctionRef(ctx, { kind, operand, name, options }) {
    let source;
    try {
        let fnArgs;
        if (operand) {
            const arg = ctx.resolve(operand);
            fnArgs = [arg];
            source =
                arg.source ||
                    String((arg.type === 'literal' && arg.value) || message_value_2.FALLBACK_SOURCE);
        }
        else {
            fnArgs = [];
            source = functionRefSourceName(kind, name);
        }
        switch (kind) {
            case 'open':
            case 'close': {
                const opt = resolveOptions(ctx, options);
                return new message_value_1.MessageMarkup(ctx, name, {
                    kind,
                    operand: fnArgs[0],
                    options: opt,
                    source
                });
            }
            default: {
                const rf = ctx.runtime[name];
                if (!rf) {
                    throw new errors_1.MessageError('missing-func', `Unknown function ${name}`);
                }
                const opt = Object.assign({ localeMatcher: ctx.localeMatcher }, resolveOptions(ctx, options));
                const res = rf(ctx.locales, opt, ...fnArgs);
                return (0, message_value_1.asMessageValue)(ctx, res, { source });
            }
        }
    }
    catch (error) {
        source ?? (source = operand ? message_value_2.FALLBACK_SOURCE : `:${name}`);
        const fb = new message_value_1.MessageFallback(ctx, { source });
        ctx.onError(error, fb);
        return fb;
    }
}
exports.resolveFunctionRef = resolveFunctionRef;
function resolveOptions(ctx, options) {
    if (!options?.length)
        return undefined;
    const opt = {};
    for (const { name, value } of options) {
        opt[name] = ctx.resolve(value).value;
    }
    return opt;
}
