import type { Context } from '../format-context';
import type { MessageValue } from '../message-value';
import { FunctionRef } from './function-ref';
import { Junk } from './junk';
import { Literal, Text } from './literal';
import { Reserved } from './reserved';
import { VariableRef } from './variable-ref';
export { isFunctionRef, FunctionRef, Option } from './function-ref';
export { isJunk, Junk } from './junk';
export { isLiteral, isText, Literal, Text } from './literal';
export { isReserved, Reserved } from './reserved';
export { isVariableRef, VariableRef } from './variable-ref';
/**
 * Wrapper for non-literal content.
 *
 * @beta
 */
export interface Expression {
    type: 'expression';
    body: Literal | VariableRef | FunctionRef | Reserved | Junk;
}
/**
 * Type guard for {@link Expression} pattern elements
 *
 * @beta
 */
export declare const isExpression: (part: any) => part is Expression;
/**
 * The contents of a message are a sequence of pattern elements, which may be
 * immediately defined literal values, a reference to a value that depends on
 * another message, the value of some runtime variable, or some function
 * defined elsewhere.
 *
 * @remarks
 * Depending on the syntax, pattern elements may be wrapped within an Expression.
 *
 * @beta
 */
export type PatternElement = Expression | FunctionRef | Junk | Literal | Reserved | Text | VariableRef;
/** @internal */
export declare function resolvePatternElement(ctx: Context, elem: PatternElement): MessageValue;
