"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolvePatternElement = exports.isExpression = exports.isVariableRef = exports.isReserved = exports.isText = exports.isLiteral = exports.isJunk = exports.isFunctionRef = void 0;
const function_ref_1 = require("./function-ref");
const junk_1 = require("./junk");
const literal_1 = require("./literal");
const reserved_1 = require("./reserved");
const variable_ref_1 = require("./variable-ref");
var function_ref_2 = require("./function-ref");
Object.defineProperty(exports, "isFunctionRef", { enumerable: true, get: function () { return function_ref_2.isFunctionRef; } });
var junk_2 = require("./junk");
Object.defineProperty(exports, "isJunk", { enumerable: true, get: function () { return junk_2.isJunk; } });
var literal_2 = require("./literal");
Object.defineProperty(exports, "isLiteral", { enumerable: true, get: function () { return literal_2.isLiteral; } });
Object.defineProperty(exports, "isText", { enumerable: true, get: function () { return literal_2.isText; } });
var reserved_2 = require("./reserved");
Object.defineProperty(exports, "isReserved", { enumerable: true, get: function () { return reserved_2.isReserved; } });
var variable_ref_2 = require("./variable-ref");
Object.defineProperty(exports, "isVariableRef", { enumerable: true, get: function () { return variable_ref_2.isVariableRef; } });
/**
 * Type guard for {@link Expression} pattern elements
 *
 * @beta
 */
// eslint-disable-next-line @typescript-eslint/no-explicit-any
const isExpression = (part) => !!part && typeof part === 'object' && part.type === 'expression';
exports.isExpression = isExpression;
/** @internal */
function resolvePatternElement(ctx, elem) {
    switch (elem.type) {
        case 'literal':
        case 'text':
            return (0, literal_1.resolveLiteral)(elem);
        case 'expression':
            return resolvePatternElement(ctx, elem.body);
        case 'variable':
            return (0, variable_ref_1.resolveVariableRef)(ctx, elem);
        case 'function':
            return (0, function_ref_1.resolveFunctionRef)(ctx, elem);
        case 'reserved':
            return (0, reserved_1.resolveReserved)(ctx, elem);
        case 'junk':
            return (0, junk_1.resolveJunk)(ctx, elem);
        default:
            // @ts-expect-error - should never happen
            throw new Error(`Unsupported pattern element: ${elem.type}`);
    }
}
exports.resolvePatternElement = resolvePatternElement;
