"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveReserved = exports.isReserved = void 0;
const errors_js_1 = require("../errors.js");
const index_js_1 = require("../message-value/index.js");
const message_value_1 = require("../message-value/message-value");
/**
 * Type guard for {@link Reserved} pattern elements
 *
 * @beta
 */
// eslint-disable-next-line @typescript-eslint/no-explicit-any
const isReserved = (part) => !!part && typeof part === 'object' && part.type === 'reserved';
exports.isReserved = isReserved;
function resolveReserved(ctx, { operand, sigil, source }) {
    if (operand) {
        const arg = ctx.resolve(operand);
        source =
            arg.source ||
                String((arg.type === 'literal' && arg.value) || message_value_1.FALLBACK_SOURCE);
    }
    const fb = new index_js_1.MessageFallback(ctx, { source });
    ctx.onError(new errors_js_1.MessageError('reserved', `Reserved ${sigil} expression is not supported`), fb);
    return fb;
}
exports.resolveReserved = resolveReserved;
